"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const events_1 = require("events");
const constants_1 = require("../constants");
const deviceinfo_1 = require("../deviceinfo");
const utils_1 = require("../utils");
const topic_1 = require("../topic");
const services_1 = require("../services");
class DeviceClient extends events_1.EventEmitter {
    constructor(TransportCtor, options) {
        super();
        this._validateClientOptions(options);
        const { productKey, productSecret, deviceKey, deviceSecret, secureMode } = options;
        this._deviceInfo = new deviceinfo_1.DeviceInfo({ productKey, productSecret, deviceKey, deviceSecret });
        this._transport = new TransportCtor(options);
        this._secureMode = secureMode;
        this.deviceData = new services_1.UpstreamService(this, this._transport);
        this.deviceCommand = new services_1.CommandService(this, this._transport);
        this._transport.on('connect', () => { this.emit('connect'); });
        this._transport.on('error', (err) => { this.emit('error', err); });
        this._transport.on('close', () => { this.emit('close'); });
        this._transport.on('end', () => { this.emit('end'); });
        this._transport.on('message', (topic, payload) => {
            this.emit('message', topic, payload);
        });
    }
    _validateClientOptions(options) {
        assert_1.default(options.brokerUrl, 'option brokerUrl is required');
        assert_1.default(options.secureMode, 'option secureMode is required');
        assert_1.default(options.secureMode in constants_1.SECURE_MODE, 'option secureMode is invalid');
        if (options.secureMode === constants_1.SECURE_MODE.VIA_DEVICE_SECRET) {
            assert_1.default(options.productKey && options.deviceKey && options.deviceSecret, 'options deviceKey & productKey & deviceSecret are all required with VIA_DEVICE_SECRET mode enabled');
        }
        if (options.secureMode === constants_1.SECURE_MODE.VIA_PRODUCT_SECRET) {
            assert_1.default(options.productKey && options.deviceKey && options.productSecret, 'options deviceKey & productKey & productSecret are all required with VIA_PRODUCT_SECRET mode enabled');
        }
    }
    get connected() {
        return this._transport.connected;
    }
    get reconnected() {
        return this._transport.reconnected;
    }
    get deviceInfo() {
        return this._deviceInfo;
    }
    get secureMode() {
        return this._secureMode;
    }
    /**
     * Connect to the cloud
     *
     * After the connection is successful, return the device secret of the device
     */
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            return utils_1.callbackToPromise((_callback) => {
                if (this._transport.connected) {
                    _callback(null, this._deviceInfo.deviceSecret);
                }
                else {
                    this._transport.connect((err) => {
                        if (err) {
                            _callback(err);
                        }
                        else if (this._secureMode === constants_1.SECURE_MODE.VIA_PRODUCT_SECRET) {
                            // 当鉴权方式为一型一密的动态激活时，需要监听Cloud返回已获取deviceSecret
                            this._onDynamicDeviceActivate(_callback);
                        }
                        else {
                            _callback(null, this._deviceInfo.deviceSecret);
                        }
                    });
                }
            });
        });
    }
    /**
     * Closes the transport connection and destroys the client resources.
     *
     * *Note: After calling this method the Client object cannot be reused.*
     */
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            return utils_1.callbackToPromise((_callback) => {
                this._transport.end(false, () => {
                    _callback();
                });
            });
        });
    }
    /**
     * Subscribe dynamic activate command of the device
     *
     * Once activated, cloud will return the device secret of the device
     */
    _onDynamicDeviceActivate(callback) {
        const deviceActivateTopic = {
            requestTopic: '/ext/session/%s/%s/thing/activate/info'
        };
        const topic = new topic_1.Topic(deviceActivateTopic, this._deviceInfo);
        this._transport.onCommand(topic, (request, response) => {
            if (request.params && request.params.deviceSecret) {
                this._deviceInfo.deviceSecret = request.params.deviceSecret;
                callback(null, request.params.deviceSecret);
            }
            response.send(200);
        });
    }
}
exports.DeviceClient = DeviceClient;
