import Service from '../service';
import { ICommandCallback } from '../../interfaces';
import { GatewayClient } from '../../client/gateway_client';
import { IUpstreamOptions } from '../../client/device_client';
import { IUpstreamResponse } from '../../message';
import { SubDeviceInfo } from '../../deviceinfo';
export default class GatewayService extends Service<GatewayClient> {
    /**
     * Register a device
     */
    registerDevice(registerParams: IRegisterUpstreamOptions): Promise<ISubDeviceRegistryPromiseResponse>;
    /**
     * Get topological relationships of sub-devices
     */
    getTopo(queryParams?: IUpstreamOptions): Promise<IGetTopoPromiseResponse>;
    /**
     * Add topological relationships of sub-devices
     */
    addTopo(addParams: IAddTopoUpstreamOptions): Promise<IUpstreamResponse<void>>;
    /**
     * Delete topological relationships of sub-devices
     */
    deleteTopo(deleteParams: IDeleteTopoUpstreamOptions): Promise<IUpstreamResponse<void>>;
    /**
     * Connect a sub-device to EnOS Cloud
     *
     * Ensure that the sub-device to connect has been registered to the EnOS Cloud.
     * Ensure that the topological relationship with the edge has been added to the edge.
     * EnOS Cloud verifies the identity of the sub-device according to the topological relationship to identify whether the sub-device can use the edge connection.
     */
    loginSubDevice(loginParams: ILoginUpstreamOptions): Promise<ISubDeviceLoginPromiseResponse>;
    /**
     * Connect sub-devices to EnOS Cloud in batch
     *
     * Ensure that the sub-devices to connect have been registered to the EnOS Cloud.
     * Ensure that the topological relationship with the edge has been added to the edge.
     * EnOS Cloud verifies the identity of the sub-devices according to the topological relationship to identify whether the sub-devices can use the edge connection.
     */
    batchLoginSubDevice(loginParams: IBatchLoginUpstreamOptions): Promise<ISubDeviceBatchLoginPromiseResponse>;
    /**
     * Disconnect the sub-device from EnOS Cloud
     */
    logoutSubDevice(logoutParams: ILogoutUpstreamOptions): Promise<ISubDeviceSimpleInfoPromise>;
    /**
     * Called when sub-devices are enabled
     */
    onSubDevicesEnable(callback: ICommandCallback<ISubDeviceSimpleInfo, any>): void;
    /**
     * Called when sub-devices are disabled
     */
    onSubDevicesDisable(callback: ICommandCallback<ISubDeviceSimpleInfo, any>): void;
    /**
     * Called when sub-devices are deleted
     */
    onSubDevicesDelete(callback: ICommandCallback<ISubDeviceSimpleInfo, any>): void;
    private _getLoginedDeviceId;
}
/** upstream options of registering sub-devices */
interface IRegisterUpstreamOptions extends IUpstreamOptions {
    /** sub-device list */
    subDevices: {
        productKey: string;
        deviceKey?: string;
        deviceName: I18NString;
        deviceAttributes?: Directory<Object>;
        timezone: string;
        deviceDesc?: string;
    }[];
}
/** successful response of registering sub-devices */
interface ISubDeviceRegisterResponse {
    assetId: string;
    deviceKey: string;
    productKey: string;
    deviceSecret: string;
}
declare type ISubDeviceRegistryPromiseResponse = IUpstreamResponse<ISubDeviceRegisterResponse[]>;
/** Upstream options of adding topological relationships of sub-devices */
interface IAddTopoUpstreamOptions extends IUpstreamOptions {
    /** 待添加拓扑关系的子设备信息 */
    subDevices: (SubDeviceInfo | ISubDeviceCredential)[];
}
/** Upstream options of deleting topological relationships of sub-devices */
interface IDeleteTopoUpstreamOptions extends IUpstreamOptions {
    subDevices: ISubDeviceSimpleInfo[];
}
/** Upstream options of connecting a sub-device to EnOS Cloud */
interface ILoginUpstreamOptions extends IUpstreamOptions {
    /** Sub-device information to be connected */
    subDevice: SubDeviceInfo | ISubDeviceCredential;
}
/** Upstream options of connecting sub-devices to EnOS Cloud in batch */
interface IBatchLoginUpstreamOptions extends IUpstreamOptions {
    /** sub-device list to be connected */
    subDevices: (SubDeviceInfo | ISubDeviceCredential)[];
    clientId?: string;
    timestamp?: string;
}
/** Upstream options of disconnecting a sub-device from EnOS Cloud */
interface ILogoutUpstreamOptions extends IUpstreamOptions {
    subDevice: ISubDeviceSimpleInfo;
}
/** Sub-device login credentials, used to generate sub-device login information */
interface ISubDeviceCredential {
    productKey: string;
    productSecret?: string;
    deviceKey: string;
    deviceSecret: string;
    /** The identifier of the sub-device. The value can be productKey or deviceName. */
    clientId?: string;
    timestamp?: number;
    signMethod?: string;
    /** Indicates whether to clear offline information for all sub-devices, which is information that has not been received by QoS 1. */
    cleanSession?: boolean;
}
/** Response of getting topological relationships of sub-devices */
interface IGetTopoResponse {
    productKey: string;
    productName: string;
    iotId: string;
    deviceKey: string;
    deviceSecret: string;
    deviceAttributes: Directory<Object>;
    deviceName: string;
    timezone: string;
    isDST: boolean;
    topo: string;
    orgCode: string;
    createBy: string;
    gmtCreate: number;
    modifiedByOrg: string;
    modifiedBy: string;
    gmtModified: number;
    gmtActive: number;
    gmtOnline: number;
    gmtOffline: number;
    status: string;
    nodeType: number;
    dataType: number;
}
declare type IGetTopoPromiseResponse = IUpstreamResponse<IGetTopoResponse[]>;
interface ISubDeviceLoginResponse {
    assetId: string;
    deviceKey: string;
    productKey: string;
}
declare type ISubDeviceLoginPromiseResponse = IUpstreamResponse<ISubDeviceLoginResponse>;
interface ISubDeviceBatchLoginResponse {
    code: number;
    message?: string;
    loginedSubDevices: ISubDeviceLoginResponse[];
    failedSubDevices: ISubDeviceSimpleInfo[];
}
declare type ISubDeviceBatchLoginPromiseResponse = IUpstreamResponse<ISubDeviceBatchLoginResponse>;
interface ISubDeviceSimpleInfo {
    productKey: string;
    deviceKey: string;
}
declare type ISubDeviceSimpleInfoPromise = IUpstreamResponse<ISubDeviceSimpleInfo>;
interface I18NString {
    defaultValue: string;
    i18nValue?: {
        en_US?: string;
        zh_CN?: string;
    };
}
export {};
