"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const service_1 = __importDefault(require("../service"));
const topic_1 = require("../../topic");
const utils_1 = require("../../utils");
const gatewayTopics = __importStar(require("./gateway_topics"));
const constants_1 = require("../../constants");
const gateway_client_1 = require("../../client/gateway_client");
const message_1 = require("../../message");
const deviceinfo_1 = require("../../deviceinfo");
class GatewayService extends service_1.default {
    /**
     * Register a device
     */
    registerDevice(registerParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(registerParams);
            assert_1.default(registerParams && registerParams.subDevices, '"subDevices" is required');
            assert_1.default(Array.isArray(registerParams.subDevices), '"subDevices" should be an array');
            const params = this._extractParams(registerParams);
            const topic = new topic_1.Topic(gatewayTopics.registerDeviceTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, (registerParams && registerParams.subDevices) || []);
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, _callback);
            });
        });
    }
    /**
     * Get topological relationships of sub-devices
     */
    getTopo(queryParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(queryParams);
            const params = this._extractParams(queryParams);
            const topic = new topic_1.Topic(gatewayTopics.getTopoTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, {});
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, _callback);
            });
        });
    }
    /**
     * Add topological relationships of sub-devices
     */
    addTopo(addParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(addParams);
            assert_1.default(addParams && addParams.subDevices, '"subDevices" is required');
            assert_1.default(Array.isArray(addParams.subDevices), '"subDevices" should be an array');
            const subDeviceLoginInfos = [];
            (addParams.subDevices || []).forEach((subDevice) => {
                if (subDevice instanceof deviceinfo_1.SubDeviceInfo) {
                    subDeviceLoginInfos.push(subDevice.getLoginInfo());
                }
                else {
                    subDeviceLoginInfos.push(new deviceinfo_1.SubDeviceInfo(subDevice).getLoginInfo());
                }
            });
            const params = this._extractParams(addParams);
            const topic = new topic_1.Topic(gatewayTopics.addTopoTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, subDeviceLoginInfos);
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, _callback);
            });
        });
    }
    /**
     * Delete topological relationships of sub-devices
     */
    deleteTopo(deleteParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(deleteParams);
            assert_1.default(deleteParams && deleteParams.subDevices, '"subDevices" is required');
            assert_1.default(Array.isArray(deleteParams.subDevices), '"subDevices" should be an array');
            const params = this._extractParams(deleteParams);
            const topic = new topic_1.Topic(gatewayTopics.deleteTopoTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, (deleteParams && deleteParams.subDevices) || []);
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, _callback);
            });
        });
    }
    /**
     * Connect a sub-device to EnOS Cloud
     *
     * Ensure that the sub-device to connect has been registered to the EnOS Cloud.
     * Ensure that the topological relationship with the edge has been added to the edge.
     * EnOS Cloud verifies the identity of the sub-device according to the topological relationship to identify whether the sub-device can use the edge connection.
     */
    loginSubDevice(loginParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(loginParams);
            assert_1.default(loginParams && loginParams.subDevice, '"subDevice" is required');
            assert_1.default(this._client instanceof gateway_client_1.GatewayClient);
            let subDeviceInfo;
            if (loginParams.subDevice instanceof deviceinfo_1.SubDeviceInfo) {
                subDeviceInfo = loginParams.subDevice;
            }
            else {
                subDeviceInfo = new deviceinfo_1.SubDeviceInfo(loginParams.subDevice);
            }
            const params = this._extractParams(loginParams);
            const topic = new topic_1.Topic(gatewayTopics.loginSubDeviceTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, subDeviceInfo.getLoginInfo() || {});
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, (err, result) => {
                    // Cache the logined sub-device info
                    if (result && result.code === constants_1.SUCCESS_CODE && result.data) {
                        this._client.loginedSubDeviceMap.set(this._getLoginedDeviceId(result.data.productKey, result.data.deviceKey), subDeviceInfo);
                    }
                    _callback(err, result);
                });
            });
        });
    }
    /**
     * Connect sub-devices to EnOS Cloud in batch
     *
     * Ensure that the sub-devices to connect have been registered to the EnOS Cloud.
     * Ensure that the topological relationship with the edge has been added to the edge.
     * EnOS Cloud verifies the identity of the sub-devices according to the topological relationship to identify whether the sub-devices can use the edge connection.
     */
    batchLoginSubDevice(loginParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(loginParams);
            assert_1.default(loginParams && loginParams.subDevices, '"subDevices" is required');
            assert_1.default(Array.isArray(loginParams.subDevices), '"subDevices should be an array');
            const params = this._extractParams(loginParams);
            const timestamp = parseInt(loginParams.timestamp || '0') || new Date().getTime();
            const clientId = loginParams.clientId || this._client.deviceInfo.productKey;
            const signMethod = constants_1.SIGN_METHOD.SHA256;
            const requestParams = {
                clientId,
                signMethod,
                timestamp: `${timestamp}`,
                subDevices: []
            };
            const subDeviceInfoMap = new Map();
            requestParams.subDevices = loginParams.subDevices.map((subDevice) => {
                const deviceInfo = new deviceinfo_1.SubDeviceInfo({
                    timestamp,
                    clientId,
                    productKey: subDevice.productKey,
                    deviceKey: subDevice.deviceKey,
                    deviceSecret: subDevice.deviceSecret
                });
                subDeviceInfoMap.set(subDevice.deviceKey, deviceInfo);
                return {
                    sign: deviceInfo.getLoginInfo().sign,
                    secureMode: deviceInfo.getSecureMode(),
                    productKey: deviceInfo.productKey,
                    deviceKey: deviceInfo.deviceKey
                };
            });
            const topic = new topic_1.Topic(gatewayTopics.batchLoginSubDeviceTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, requestParams || {});
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, (err, result) => {
                    // Cache logined sub-devices
                    if (result && result.code === constants_1.SUCCESS_CODE && result.data) {
                        (result.data.loginedSubDevices || []).forEach((subDevice) => {
                            const subDeviceInfo = subDeviceInfoMap.get(subDevice.deviceKey);
                            if (subDeviceInfo) {
                                this._client.loginedSubDeviceMap.set(this._getLoginedDeviceId(subDevice.productKey, subDevice.deviceKey), subDeviceInfo);
                            }
                        });
                    }
                    _callback(err, result);
                });
            });
        });
    }
    /**
     * Disconnect the sub-device from EnOS Cloud
     */
    logoutSubDevice(logoutParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(logoutParams);
            assert_1.default(logoutParams && logoutParams.subDevice, '"subDevice" is required');
            const params = this._extractParams(logoutParams);
            const topic = new topic_1.Topic(gatewayTopics.logoutSubDeviceTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, (logoutParams && logoutParams.subDevice) || []);
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, (err, result) => {
                    if (result && result.code === constants_1.SUCCESS_CODE && result.data) {
                        // remove logout sub device from cache
                        this._client.loginedSubDeviceMap.delete(this._getLoginedDeviceId(result.data.productKey, result.data.deviceKey));
                    }
                    _callback(err, result);
                });
            });
        });
    }
    /**
     * Called when sub-devices are enabled
     */
    onSubDevicesEnable(callback) {
        const topic = new topic_1.Topic(gatewayTopics.subDeviceEnableTopic, this._client.deviceInfo);
        this._transport.onCommand(topic, callback);
    }
    /**
     * Called when sub-devices are disabled
     */
    onSubDevicesDisable(callback) {
        const topic = new topic_1.Topic(gatewayTopics.subDeviceDisableTopic, this._client.deviceInfo);
        this._transport.onCommand(topic, callback);
    }
    /**
     * Called when sub-devices are deleted
     */
    onSubDevicesDelete(callback) {
        const topic = new topic_1.Topic(gatewayTopics.subDeviceDeleteTopic, this._client.deviceInfo);
        this._transport.onCommand(topic, callback);
    }
    _getLoginedDeviceId(productKey, deviceKey) {
        return `${productKey}-${deviceKey}`;
    }
}
exports.default = GatewayService;
