/// <reference types="node" />
import AttributeService from '../attribute_service';
import { IDeviceUpstreamOptions } from '../../client/device_client';
import { IUpstreamResponse } from '../../message';
/** Upstream options of reporting (offline) measurement points */
interface IPostMPUpstreamOptions extends IDeviceUpstreamOptions {
    /** Information of measurement points to be reported */
    point: IMeasurePoint;
}
/** Upstream options of reporting (offline) measurement points in batch */
interface IBatchPostMPUpstreamOptions extends IDeviceUpstreamOptions {
    /** Information of measurement points to be reported in batch */
    points: IBatchMeasurePoint[];
    /**
     * Whether to allow sending measurement point data for offline devices. False by default.
     *
     * If sending data of offline device is not allowed and the request contains an offline device, the entire request is declined.
     */
    allowOfflineSubDevice?: boolean;
    /**
     * Whether to neglect invalid measurement point data in a request. False by default.
     *
     * If the value is set to false and the request contains invalid measurement point data, the entire request is declined.
     */
    skipInvalidMeasurepoints?: boolean;
}
/** Measurement point information */
declare type IMeasurePoint = {
    measurepoints: {
        [x: string]: Object;
    };
    time?: number;
};
/** Measurement point information to be reported in batch */
declare type IBatchMeasurePoint = {
    /**
     * Product key of a sub-device.
     *
     * 1. If the data of a sub-device needs reporting, the product key and device key of the sub-device are required.
     * 2. If the product key and device key of the sub-device is not provided, those of the gateway device is used instead.
     */
    productKey?: string;
    /**
     * Device key of a sub-device.
     *
     * 1. If the data of a sub-device needs reporting, the product key and device key of the sub-device are required.
     * 2. If the product key and device key of the sub-device is not provided, those of the gateway device is used
     */
    deviceKey?: string;
} & IMeasurePoint;
interface ISendEventUpstreamOptions extends IDeviceUpstreamOptions {
    eventName: string;
    eventParams: any;
}
interface IRawDataUpstreamOptions extends IDeviceUpstreamOptions {
    /** Raw data to be reported */
    rawData: Buffer;
}
export default class TSLService extends AttributeService {
    /**
     * Report device measurement points
     */
    postMeasurepoint(postParams: IPostMPUpstreamOptions): Promise<IUpstreamResponse<void>>;
    /**
     * Report offline measurement points
     */
    resumeMeasurepoint(resumeParams: IPostMPUpstreamOptions): Promise<IUpstreamResponse<void>>;
    /**
     * Report device measurement points in batch
     *
     * Suitable for the following scenarios:
     * 1. A gateway device reports the measurement point data of its sub-devices.
     * 2. A device that is directly connected to EnOS reports the measurement point data of different timestamps.
     * 3. A mixture of the previous two scenarios.
     */
    batchPostMeasurepoint(postParams: IBatchPostMPUpstreamOptions): Promise<IUpstreamResponse<void>>;
    /**
     * Report offline measurement points in batch
     *
     * Suitable for the following scenarios:
     * 1. A gateway device reports the measurement point data of its sub-devices.
     * 2. A device that is directly connected to EnOS reports the measurement point data of different timestamps.
     * 3. A mixture of the previous two scenarios.
     */
    batchResumeMeasurepoint(postParams: IBatchPostMPUpstreamOptions): Promise<IUpstreamResponse<void>>;
    /**
     * Report device events (non-passthrough)
     */
    postEvent(postEventParams: ISendEventUpstreamOptions): Promise<IUpstreamResponse<void>>;
    /**
     * Report raw data to the cloud (passthrough)
     *
     * The device sends raw data such as a binary data stream to the cloud.
     * EnOS Cloud converts the raw data to JSON format using the script that you defined. The raw data can be of any format.
     */
    upRawModel(rawDataParams: IRawDataUpstreamOptions): Promise<Buffer>;
    /**
     * Report (offline) measurement points
     */
    private _postMp;
    /**
     * Report (offline) measurepoint points in batch
     */
    private _postMpBatch;
}
export {};
