"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const tslTopics = __importStar(require("./tsl_topics"));
const utils_1 = require("../../utils");
const topic_1 = require("../../topic");
const attribute_service_1 = __importDefault(require("../attribute_service"));
const message_1 = require("../../message");
class TSLService extends attribute_service_1.default {
    /**
     * Report device measurement points
     */
    postMeasurepoint(postParams) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._postMp(postParams, tslTopics.postMeasurepointTopic);
        });
    }
    /**
     * Report offline measurement points
     */
    resumeMeasurepoint(resumeParams) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._postMp(resumeParams, tslTopics.resumeMeasurepointTopic);
        });
    }
    /**
     * Report device measurement points in batch
     *
     * Suitable for the following scenarios:
     * 1. A gateway device reports the measurement point data of its sub-devices.
     * 2. A device that is directly connected to EnOS reports the measurement point data of different timestamps.
     * 3. A mixture of the previous two scenarios.
     */
    batchPostMeasurepoint(postParams) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._postMpBatch(postParams, tslTopics.batchPostMeasurepointTopic);
        });
    }
    /**
     * Report offline measurement points in batch
     *
     * Suitable for the following scenarios:
     * 1. A gateway device reports the measurement point data of its sub-devices.
     * 2. A device that is directly connected to EnOS reports the measurement point data of different timestamps.
     * 3. A mixture of the previous two scenarios.
     */
    batchResumeMeasurepoint(postParams) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._postMpBatch(postParams, tslTopics.batchResumeMeasurepointTopic);
        });
    }
    /**
     * Report device events (non-passthrough)
     */
    postEvent(postEventParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(postEventParams);
            assert_1.default(postEventParams && postEventParams.eventName, 'eventName is required');
            const params = this._extractParams(postEventParams);
            const topic = new topic_1.Topic(tslTopics.postEventTopic, params.deviceInfo, postEventParams.eventName);
            const message = new message_1.UpstreamRequest(topic, { events: (postEventParams && postEventParams.eventParams) || {} });
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, _callback);
            });
        });
    }
    /**
     * Report raw data to the cloud (passthrough)
     *
     * The device sends raw data such as a binary data stream to the cloud.
     * EnOS Cloud converts the raw data to JSON format using the script that you defined. The raw data can be of any format.
     */
    upRawModel(rawDataParams) {
        return __awaiter(this, void 0, void 0, function* () {
            this._validateParams(rawDataParams);
            assert_1.default(rawDataParams && rawDataParams.rawData, 'rawData is required');
            const params = this._extractParams(rawDataParams);
            const topic = new topic_1.Topic(tslTopics.modelUpRawTopic, params.deviceInfo);
            const message = new message_1.UpstreamRequest(topic, rawDataParams.rawData, true);
            return utils_1.callbackToPromise((_callback) => {
                this._transport.publish(message, params.upstreamOptions, _callback);
            });
        });
    }
    /**
     * Report (offline) measurement points
     */
    _postMp(postParams, mpTopic) {
        this._validateParams(postParams);
        assert_1.default(postParams && postParams.point && postParams.point.measurepoints, 'point.measurepoints is required');
        const params = this._extractParams(postParams);
        const topic = new topic_1.Topic(mpTopic, params.deviceInfo);
        const message = new message_1.UpstreamRequest(topic, ((postParams && postParams.point) || {}));
        return utils_1.callbackToPromise((_callback) => {
            this._transport.publish(message, params.upstreamOptions, _callback);
        });
    }
    /**
     * Report (offline) measurepoint points in batch
     */
    _postMpBatch(postParams, batchTopic) {
        this._validateParams(postParams);
        assert_1.default(postParams && postParams.points, 'points is required');
        assert_1.default(Array.isArray(postParams.points), 'points should be array');
        for (let i = 0; i < postParams.points.length; i++) {
            const point = postParams.points[i];
            assert_1.default(point.measurepoints, 'measurepoints is required');
        }
        const params = this._extractParams(postParams);
        const topic = new topic_1.Topic(batchTopic, params.deviceInfo);
        const message = new message_1.MeasurePointBatchRequest(topic, ((postParams && postParams.points) || []));
        if (postParams.allowOfflineSubDevice) {
            message.setAllowOfflineSubDevice(postParams.allowOfflineSubDevice);
        }
        if (postParams.skipInvalidMeasurepoints) {
            message.setSkipInvalidMeasurepoints(postParams.skipInvalidMeasurepoints);
        }
        return utils_1.callbackToPromise((_callback) => {
            this._transport.publish(message, params.upstreamOptions, _callback);
        });
    }
}
exports.default = TSLService;
